package com.link.core.basic.service.snowflake;

import com.link.core.basic.service.BasicServiceException;
import com.link.core.cllog.LCLogger;
import com.link.core.util.CoreYmlConstants;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.redisclient.LinkRedisClient;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class GenerateCodeSingleton {

   private static final Logger logger = LogManager.getLogger(GenerateCodeSingleton.class);
   private static volatile GenerateCodeSingleton instance;
   private static final List KEYS_ORDER_CODE = Collections.singletonList("{order-code}");
   private static AtomicLong orderSequence = new AtomicLong(1L);
   private static AtomicLong orderRedisSequence;
   private static final List KEYS_MARKET_ACT_CODE = Collections.singletonList("{market-act-code}");
   private static AtomicLong marketActSequence = new AtomicLong(1L);
   private static AtomicLong marketActRedisSequence;
   private static long sequenceDigit;
   private static long dateDigit;
   private static AtomicLong claimScanSequence = new AtomicLong(1L);
   private static AtomicLong claimScanRedisSequence;
   private static final List KEYS_CLAIM_SCAN_CODE = Collections.singletonList("{claim-scan-code}");
   private static AtomicLong verifyScanSequence = new AtomicLong(1L);
   private static AtomicLong verifyScanRedisSequence;
   private static final List KEYS_VERIFY_SCAN_CODE = Collections.singletonList("{verify-scan-code}");


   public static GenerateCodeSingleton getInstance() {
      if(null == instance) {
         Class var0 = GenerateCodeSingleton.class;
         synchronized(GenerateCodeSingleton.class) {
            if(null == instance) {
               instance = new GenerateCodeSingleton();
            }
         }
      }

      return instance;
   }

   private GenerateCodeSingleton() {
      if(null != CoreYmlConstants.getLinkCodeDigit() && 3 <= CoreYmlConstants.getLinkCodeDigit().intValue() && CoreYmlConstants.getLinkCodeDigit().intValue() <= 5) {
         try {
            sequenceDigit = (long)Math.pow(10.0D, (double)CoreYmlConstants.getLinkCodeDigit().intValue());
            dateDigit = (long)Math.pow(10.0D, (double)(CoreYmlConstants.getLinkCodeDigit().intValue() + 3));
         } catch (Exception var2) {
            LCLogger.error().withMessageKey("GenerateCodeSingleton()").withMessage("编号生成类初始化出现异常，自身序列上限设为默认1000").flush();
            sequenceDigit = 1000L;
            dateDigit = 1000000L;
         }
      } else {
         sequenceDigit = 1000L;
         dateDigit = 1000000L;
      }

   }

   public synchronized Long getOrderCodeLong() {
      LinkRedisClient jedis = null;
      Long result = Long.valueOf(0L);

      try {
         orderSequence.compareAndSet(sequenceDigit, 1L);
         if(orderSequence.get() == 1L) {
            jedis = RedisUtil.getJedis();
            orderRedisSequence = new AtomicLong(((Long)jedis.eval(this.getLuaScript(), KEYS_ORDER_CODE, new ArrayList())).longValue());
         }

         long e = Long.parseLong(getToday("yyMMdd"));
         result = Long.valueOf(e * dateDigit + orderRedisSequence.get() * sequenceDigit + orderSequence.getAndIncrement());
      } catch (Exception var8) {
         LCLogger.error().withMessageKey("getOrderCodeLong").withMessage("生成编号异常：" + var8.getMessage()).flush();
         throw new BasicServiceException("获取编号出现异常，请联系管理员！");
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return result;
   }

   public String getOrderCodeStr() {
      return this.getOrderCodeLong().toString();
   }

   public synchronized Long getMarketActCodeLong() {
      LinkRedisClient jedis = null;
      Long result = Long.valueOf(0L);

      try {
         marketActSequence.compareAndSet(sequenceDigit, 1L);
         if(marketActSequence.get() == 1L) {
            jedis = RedisUtil.getJedis();
            marketActRedisSequence = new AtomicLong(((Long)jedis.eval(this.getLuaScript(), KEYS_MARKET_ACT_CODE, new ArrayList())).longValue());
         }

         long e = Long.parseLong(getToday("yyMMdd"));
         result = Long.valueOf(e * dateDigit + marketActRedisSequence.get() * sequenceDigit + marketActSequence.getAndIncrement());
      } catch (Exception var8) {
         LCLogger.error().withMessageKey("getMarketActCodeLong").withMessage("生成市场活动编号异常：" + var8.getMessage()).flush();
         throw new BasicServiceException("获取市场活动编号出现异常，请联系管理员！");
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return result;
   }

   public String getMarketActCodeStr() {
      return "SCHD".concat(this.getMarketActCodeLong().toString());
   }

   private String getLuaScript() {
      StringBuilder sb = new StringBuilder();
      sb.append("local key = \"code.index:\" .. KEYS[1]\n");
      sb.append("local exist = redis.call(\"EXISTS\", key)\n");
      sb.append("if exist == 1 then\n");
      sb.append("\tlocal next = tonumber(redis.call(\"INCR\", key))\n");
      sb.append("\tif next % 1000 == 0 then\n");
      sb.append("\t\tredis.call(\"SET\", key, 0)\n");
      sb.append("\t\treturn 0\n");
      sb.append("\telse\n");
      sb.append("\t\treturn next\n");
      sb.append("\tend\n");
      sb.append("else\n");
      sb.append("\tredis.call(\"SET\", key, 0)\n");
      sb.append("\treturn 0\n");
      sb.append("end\n");
      return sb.toString();
   }

   public static String getToday(String fmt) {
      if(StringUtils.isBlank(fmt)) {
         fmt = "yyMMdd";
      }

      Date date = new Date();
      SimpleDateFormat dateFormat = new SimpleDateFormat(fmt);
      String today = dateFormat.format(date);
      return today;
   }

   public static String getYearAndMonth() {
      Date date = new Date();
      SimpleDateFormat dateFormat = new SimpleDateFormat("yyMM");
      return dateFormat.format(date);
   }

   public static void main(String[] args) {
      StringBuilder sb = new StringBuilder();
      sb.append("local key = \"{market-act-code}\"\n");
      sb.append("local exist = redis.call(\"EXISTS\", key)\n");
      sb.append("if exist == 1 then\n");
      sb.append("\tlocal next = tonumber(redis.call(\"INCR\", key))\n");
      sb.append("\tif next % 1000 == 0 then\n");
      sb.append("\t\tredis.call(\"SET\", key, 0)\n");
      sb.append("\t\treturn 0\n");
      sb.append("\telse\n");
      sb.append("\t\treturn next\n");
      sb.append("\tend\n");
      sb.append("else\n");
      sb.append("\tredis.call(\"SET\", key, 0)\n");
      sb.append("\treturn 0\n");
      sb.append("end\n");
      System.out.println(sb.toString());
   }

   public synchronized Long getClaimScanCodeLong() {
      LinkRedisClient jedis = null;
      Long result = Long.valueOf(0L);

      try {
         claimScanSequence.compareAndSet(sequenceDigit, 1L);
         if(claimScanSequence.get() == 1L) {
            jedis = RedisUtil.getJedis();
            claimScanRedisSequence = new AtomicLong(((Long)jedis.eval(this.getLuaScript(), KEYS_CLAIM_SCAN_CODE, new ArrayList())).longValue());
         }

         long e = Long.parseLong(getToday("yyMMdd"));
         result = Long.valueOf(e * dateDigit + claimScanRedisSequence.get() * sequenceDigit + claimScanSequence.getAndIncrement());
      } catch (Exception var8) {
         LCLogger.error().withMessageKey("getClaimScanCodeLong").withMessage("生成领用扫码编号异常：" + var8.getMessage()).flush();
         throw new BasicServiceException("获取领用扫码编号出现异常，请联系管理员！");
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return result;
   }

   public String getClaimScanCodeStr() {
      return "LYSM".concat(this.getClaimScanCodeLong().toString());
   }

   public synchronized Long getVerifyScanCodeLong() {
      LinkRedisClient jedis = null;
      Long result = Long.valueOf(0L);

      try {
         verifyScanSequence.compareAndSet(sequenceDigit, 1L);
         if(verifyScanSequence.get() == 1L) {
            jedis = RedisUtil.getJedis();
            verifyScanRedisSequence = new AtomicLong(((Long)jedis.eval(this.getLuaScript(), KEYS_VERIFY_SCAN_CODE, new ArrayList())).longValue());
         }

         long e = Long.parseLong(getToday("yyMMdd"));
         result = Long.valueOf(e * dateDigit + verifyScanRedisSequence.get() * sequenceDigit + verifyScanSequence.getAndIncrement());
      } catch (Exception var8) {
         LCLogger.error().withMessageKey("getClaimScanCodeLong").withMessage("生成核销扫码编号异常：" + var8.getMessage()).flush();
         throw new BasicServiceException("获取核销扫码编号出现异常，请联系管理员！");
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return result;
   }

   public String getVerifyScanCodeStr() {
      return "HXSM".concat(this.getVerifyScanCodeLong().toString());
   }

}
